/*
    by Luigi Auriemma
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <sys/stat.h>

typedef uint8_t     u8;
typedef uint16_t    u16;
typedef uint32_t    u32;



#define VER     "0.1"



void loadrunner_crypt(u32 seed, u8 *pass, int passlen);
u8 hex2byte(u8 *hex);



int main(int argc, char *argv[]) {
    u32     seed;
    int     i,
            len;
    u8      *pass;

    setbuf(stdout, NULL);

    fputs("\n"
        "HP LoadRunner password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stderr);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <encrypted_password>\n"
            "\n", argv[0]);
        exit(1);
    }

    pass = strdup(argv[1]);
    for(i = 0; pass[i << 1]; i++) {
        pass[i] = hex2byte(pass + (i << 1));
    }
    len = i;

    if(len < 4) {
        printf("\nError: input password is too short\n");
        exit(1);
    }
    seed = (pass[0] << 24) | (pass[1] << 16) | (pass[2] << 8) | (pass[3]);
    printf("- seed 0x%08x\n", seed);

    len -= 4;
    memmove(pass, pass + 4, len);
    loadrunner_crypt(seed, pass, len);

    printf("- password: %.*s\n", len, pass);
    return(0);
}



    // the algorithm is the same for both encryption and decryption
void loadrunner_crypt(u32 seed, u8 *pass, int passlen) {
    int     i,
            j;
    u8      chr;

    for(i = 0; i < passlen; i++) {
        chr = pass[i];

        for(j = 7; j >= 0; j--) {
            if(!(seed & 1)) {
                seed >>= 1;
                continue;
            }
            seed >>= 1;
            seed ^= 0x87654255;
            chr  ^= 1 << j;
        }
        pass[i] = chr;
    }
}



u8 hex2byte(u8 *hex) {
    static const u8 hextable[256] =
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x00\x00\x00\x00\x00\x00"
        "\x00\x0a\x0b\x0c\x0d\x0e\x0f\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x0a\x0b\x0c\x0d\x0e\x0f\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";

    return((hextable[hex[0]] << 4) | hextable[hex[1]]);
}


